﻿local LFM = LFMonitor
local cfflag, ddmenu, clmenu, pname, pflag, itemlink
local mouse, buttons = false, false
LFM.inactive, LFM.combat = false, false
local tradechannel = -1
local _G = _G
local events = {}
local matchedval

local function sendWhisper(priority, prefix, msg, type, lang, dest)
	local ch, msgchunk, segment, m
	local space = 0
	local msgs = {}
	msgchunk = msg
	if string.len(msg) > 255 then
		while string.len(msgchunk) > 255 do
			segment = string.sub(msgchunk, 1, 255)
			for ch = 255, 1, -1 do
				if string.sub(segment, ch, ch) == " " then
					space = ch
					break
				end
			end
			if space == 0 then space = 255 end
			table.insert(msgs, string.sub(segment, 1, space - 1))
			msgchunk = string.sub(msgchunk, space + 1)
		end
	end
	table.insert(msgs, msgchunk)
	for _, m in ipairs(msgs) do SendChatMessage(priority, prefix, m, type, lang, dest) end
end

local function resetEvent()
	if LFM.alertframe:GetScript("OnUpdate") == nil then
		LFM.alertframe:SetScript("OnUpdate", function(this, elapsed) LFM:AlertOnUpdate(this, elapsed) end)
	end
end

function LFM:PlayAlert(id)
	local sound = LFM.sounds[id]
	if LFM.db.profile.options.soundon == 1 then
		if sound.type == "game" then PlaySound(sound.file)
		else PlaySoundFile("Interface\\AddOns\\LFM\\Sounds\\" .. sound.file) end
	end
end

local function GetAutoReply()
	local reply
	if matchedvalue then reply = LFM.db.char.responses[matchedvalue] end
	return reply
end

function LFM:ChatCommand(input)
	local command = string.lower(input or "")
	if command == "show" then
		LFM.db.profile.options.disabled = false
		SetDisable(false)
	elseif command == "hide" then
		LFM.db.profile.options.disabled = true
		SetDisable(true)
	else InterfaceOptionsFrame_OpenToCategory(_G.LFMOptionsMain) end
end

local function GetLocalTime()
	local format = LFM.timestamps[LFM.db.profile.options.timestamp].format
	local localtime = date(format)
	return localtime
end

local function IsInstance()
	local inInstance, instanceType = IsInInstance()
	local ininstance = false
	if inInstance ~= nil then
		if instanceType ~= "none" then ininstance = true end
	end
  	return ininstance
end

local function alertHide() LFM.alertframe:Hide() end

function LFM:lfmframeOnUpdate(this, elapsed)
	local flash = LFMonitorEndFlash
	local flashtimer = LFM.frame.flashtimer + elapsed
	if (LFM.alertframe:AtBottom()) then
		if (flash:IsShown()) then flash:Hide() end
		LFM.frame:SetScript("OnUpdate", nil)
	elseif (flashtimer < _G.CHAT_BUTTON_FLASH_TIME ) then
		LFM.frame.flashtimer = flashtimer
	else
		LFM.frame.flashtimer = 0
		if (flash:IsShown()) then flash:Hide()
		else flash:Show() end
	end
end

function LFM:GetChannelNumber(channel)
	local channels = {GetChannelList()}
	local channelnumber = -1
	for num = 1, #channels, 2 do
		if (channels[num + 1] == channel) then channelnumber = num end
	end
	return channelnumber
end

local function initevents(init)
	if LFM.testmode then LFM.frame:RegisterEvent("CHAT_MSG_WHISPER_INFORM")
	else
		LFM.frame:RegisterEvent("CHAT_MSG_CHANNEL")
		LFM.frame:RegisterEvent("CHAT_MSG_SAY")
		LFM.frame:RegisterEvent("CHAT_MSG_YELL")
	end
	if not LFM.inactive then
		LFM.frame:RegisterEvent("PLAYER_UPDATE_RESTING")
		LFM.frame:RegisterEvent("PLAYER_ENTERING_WORLD")
		LFM.frame:RegisterEvent("PLAYER_REGEN_DISABLED")
		LFM.frame:RegisterEvent("PLAYER_REGEN_ENABLED")
		LFM.frame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
		LFM.frame:RegisterEvent("ZONE_CHANGED")
		LFM.frame:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE")
	end
	LFM.inactive = false
	LFM.frame:SetScript("OnEvent", function(...) LFM:eventresponder(...) end)
	if init then
		tradechannel = LFM:GetChannelNumber("Trade")
		LFM:DisableCheck()
	end
	LFM.alertframe:Clear()
end

local function SetDisable(disable)
	if disable then
		if LFM.alertframe:IsVisible() then
			LFM:OnDisable()
			LFM.inactive = true
		end
	else
		if not LFM.alertframe:IsVisible() then
			UIFrameFadeIn(LFM.alertframe, 1)
			resetEvent()
			initevents()
			LFM.inactive = false
		end
	end
end

function LFM:DisableCheck()
	local disable = false
	tradechannel = LFM:GetChannelNumber("Trade")
	if LFM.db.profile.options.hinst then
		if IsInstance() then disable = true end
	end
	if LFM.db.profile.options.rested then
		if not IsResting() then disable = true end
	end
	if LFM.db.profile.options.trade then
		if tradechannel == -1 then disable = true end
	end
	SetDisable(disable)
end

function LFM:IsInBounds()
	local scale = UIParent:GetEffectiveScale()
	local x, y = GetCursorPosition()
	local xmin = LFM.db.profile.alert.position.x
	local xmax = xmin + LFM.db.profile.alert.position.width
	local ymax = LFM.db.profile.alert.position.y
	local ymin = ymax - LFM.db.profile.alert.position.height
	local inbounds = nil
	x = x / scale
	y = y / scale
	if (x >= xmin) and (x <= xmax) and (y >= ymin) and (y <= ymax) then inbounds = true end
	return inbounds
end

function LFM:AlertShowButtons()
	UIFrameFadeIn(LFMonitorAlertFrameUpBtn, 0.5, 0, 1)
	UIFrameFadeIn(LFMonitorAlertFrameDownBtn, 0.5, 0, 1)
	UIFrameFadeIn(LFMonitorAlertFrameEndBtn, 0.5, 0, 1)
end

function LFM:AlertHideButtons()
	LFMonitorAlertFrameUpBtn:Hide()
	LFMonitorAlertFrameDownBtn:Hide()
	LFMonitorAlertFrameEndBtn:Hide()
end

local function checkHistory(msg, sender)
--ignore duplicate messages sent within 20 seconds of each other
--to try and cut down on channel spam
	local i, v
	local valid = 1
	if #LFM.history > 0 then
		for _, v in pairs(LFM.history) do
			if v.msender == sender and v.mmsg == msg then
				if GetTime() - v.mtime < 20 then valid = nil end
			end
		end
	end
	if valid then
		if LFM.hindex == 10 then LFM.hindex = 1 end
		table.insert(LFM.history, LFM.hindex, {msender = sender, mmsg = msg, mtime = GetTime()})
		LFM.hindex = LFM.hindex + 1
	end
	return valid
end

local function addAlert(msg, sender, guid)
	local hlink
	local plink = "[" .. sender .. "]:|h|r "
	local tlink = "|c" .. LFM.db.profile.alert.font.timestamp .. GetLocalTime() .. "|r "
	local mcol = "|c" .. LFM.db.profile.alert.font.text
	local pcol

	if guid == -99999 then return end
	if LFM.db.profile.options.useclasscolours then pcol = LFM:GetClassColour(guid)
	else pcol = LFM.db.profile.alert.font.player end

	hlink = "|c" .. pcol .. "|Hplayer:" .. sender .. ":LFM:" .. cfflag .. "|h"	

	if checkHistory(msg, sender) then
		LFM:PlayAlert(LFM.db.profile.options.sound)
		if LFM.alertframe:GetAlpha() ~= 1 then LFM:AlertFadeIn(0.1) end
		LFM.alertframe.TimeSinceLastUpdate = 0
		LFM.alertframe:AddMessage(tlink .. hlink .. plink .. mcol .. LFM:FormatMessage(msg) .. "|r", 1, 1, 1, 1, 60.0)
	end
end

function LFM:SetLocked(locked)
	LFM.alertframe.isLocked = locked
	LFM.db.profile.options.locked = locked
end

function LFM:AlertFadeIn(ftime)
	if not mouse then
		if LFM.alertframe:GetAlpha() ~= 1 then
			UIFrameFadeIn(LFM.alertframe, ftime, 0.3, 1)
			resetEvent()
		end
	end
end

function LFM:AlertFadeOut(ftime)
	if LFM.alertframe:GetAlpha() == 1 then
		UIFrameFadeOut(LFM.alertframe, ftime, 1, 0.3)
	end
end

function LFM:SetMouse(bValue) mouse = bValue end

function LFM:SetAlertFont()
	local r, g, b, a = LFM:HexToRGB(LFM.db.profile.alert.font.text)
	local fontfile = LFM.fonts[LFM.db.profile.alert.font.index].file
	LFM.alertframe:SetFont(fontfile, LFM.db.profile.alert.font.height)	
end

function LFM:SetAlertBackdropColour()
	local r, g, b, a = LFM:HexToRGB(LFM.db.profile.alert.backdrop.background)
	LFM.alertframe:SetBackdropColor(r, g, b, a)
end

function LFM:SetAlertBorderColour()
	local r, g, b, a = LFM:HexToRGB(LFM.db.profile.alert.backdrop.border)
	LFM.alertframe:SetBackdropBorderColor(r, g, b, a)
end

local function unlink(msg)
	--replace links with the link text
	local pre, linktext, post = string.match(msg, "(.*)|c%x+|H%a+:.+|h%[(.+)%]|h|r(.*)")
	local unlinked

	pre = pre or ""
	post = post or ""

	if linktext then unlinked = string.lower(pre .. linktext .. post) end
	return unlinked or string.lower(msg)
end

local function doSearch(profTxt, msgTxt, typeTxt, sender, guid, omsg)
	local lmsg = unlink(msgTxt)
	local pattern, message, pattern2
	local matched
	local match2

	pattern = ".*(www%.).*"
	message = string.match(lmsg, pattern)
	matchedvalue = nil
	
	if message then return false end --found probable spam, so ignore

	for _, stxt in ipairs(profTxt) do
		for _, ttxt in ipairs(typeTxt) do
			pattern = "^" .. ttxt .. "(%s+.*" ..  stxt .. "%p*%s*.*)"
			pattern2 = ".*%s+" .. ttxt .. "(%s+.*" .. stxt .. "%p*%s*.*)"
			if ((typeTxt[1] == LFM.lfwtxt[1]) or (cfflag == "CUSTOM")) then pattern2 = ttxt end
			message = string.match(lmsg, pattern)
			if not message then message = string.match(lmsg, pattern2) end
			--make sure we've not matched only part of a word
			if message then
				if stxt == "" then stxt = ".*" end
				pattern = "^.*%s+" .. stxt .. "%p*%s.*"
				pattern2 = "^.*%s+" .. stxt .. "%p*$"
				if typeTxt[1] == LFM.lfwtxt[1] or cfflag == "CUSTOM" then pattern = ttxt end
				match2 = string.match(message, pattern)
				if not match2 then match2 =string.match(message, pattern2) end
				if match2 then
					addAlert(omsg, sender, guid)
					matched = true
					matchedvalue = profTxt[1]
					break
				end
			end
		end
		if matched then break end
	end
	return matched
end

function LFM:OnEvent(this, event, arg1, ...)
	if event == "ADDON_LOADED" then
		if arg1 == "LFM" then
			SLASH_LFM_CMD1 = "/lfmonitor"
			SLASH_LFM_CMD2 = "/lfmon"
			SlashCmdList["LFM_CMD"] = function(input) LFM:ChatCommand(input) end
		elseif arg1 == "TradeForwarder" then LFM.tfw = true end
	elseif (event == "PLAYER_LOGIN") then
		LFM.db = LFM.LS("AceDB-3.0"):New("LFMonitorDB", LFM.defaults, true)
		LFM:CreateOptions()
		if LFM.db.profile.alert.position.x == -1 then
			local x = (UIParent:GetWidth() - LFM.db.profile.alert.position.width) / 2
			local y = (UIParent:GetHeight() - LFM.db.profile.alert.position.height) / 2
			LFM.db.profile.alert.position.x = x
			LFM.db.profile.alert.position.y = y
		end
		LFM.alertframe = LFM:CreateAlertFrame()
		initevents(true)
		LFM:BuildTables()
		tradechannel = LFM:GetChannelNumber("Trade")
		LFM:DisableCheck()	
	end
end

function LFM:eventresponder(this, event, ...) if (events[event]) then events[event](event, ...) end end

function LFM:OnDisable()
	local r, g, b, a = LFM.alertframe:GetBackdropColor()
	if LFM.testmode then LFM.frame:UnregisterEvent("CHAT_MSG_WHISPER_INFORM")
	else
		LFM.frame:UnregisterEvent("CHAT_MSG_CHANNEL")
		LFM.frame:UnregisterEvent("CHAT_MSG_SAY")
		LFM.frame:UnregisterEvent("CHAT_MSG_YELL")
	end

	LFM.frame:SetScript("OnUpdate", nil)
	LFM.alertframe:SetScript("OnUpdate", nil)
	UIFrameFadeOut(LFM.alertframe, 1, a, 0)
	LFM.alertframe.fadeInfo.finishedFunc = alertHide
	LFM.inactive = true
end

function events.PLAYER_REGEN_DISABLED()
	if LFM.db.profile.options.LFM.combat then
		if LFM.alertframe:IsVisible() then
			dis = true
			LFM.combat = true
			UIFrameFadeOut(LFM.alertframe, 1)
			LFM.alertframe.fadeInfo.finishedFunc = alertHide
		end
	end
end

function events.PLAYER_REGEN_ENABLED()
	if LFM.combat then
		UIFrameFadeIn(LFM.alertframe, 1)
		resetEvent()
		LFM.combat = false
	end
end

function events.PLAYER_UPDATE_RESTING()
	tradechannel = LFM:GetChannelNumber("Trade")
	LFM:DisableCheck()
end

function events.ZONE_CHANGED_NEW_AREA()
	tradechannel = LFM:GetChannelNumber("Trade")
	LFM:DisableCheck()
end

function events.ZONE_CHANGED()
	tradechannel = LFM:GetChannelNumber("Trade")
	LFM:DisableCheck()
end

function events.PLAYER_ENTERING_WORLD()
	tradechannel = LFM:GetChannelNumber("Trade")
	LFM:DisableCheck()
	if LFM.db.char.raids.eoe then LFM.db.char.raids = {} end
end

function LFM:AlertOnUpdate(this, elapsed)
	this.TimeSinceLastUpdate = this.TimeSinceLastUpdate + elapsed
	if this.TimeSinceLastUpdate < LFM.LFM_Update_Int then return end
	if LFM.alertframe:IsVisible() then
		if LFM.alertframe:GetAlpha() == 1 then 
			if not mouse then
				this.TimeSinceLastUpdate = 0
				LFM:AlertHideButtons()
		    	LFM:AlertFadeOut(2)
		    	LFM:AlertFadeOut(2)
				LFM.alertframe:SetScript("OnUpdate", nil)
				mouse = false
		  	end
		end
	end
end

function LFM:OnHyperlinkShow(this, link, text, button) SetItemRef(link, text, button) end

--function to add spaces around {star},{square}, etc so they are parsed correctly
local function formatspecial(msg)
	local out = string.gsub(msg, "{", " {")
	out = string.gsub(out, "}", "} ")
	return out
end

local function ChatParser(event, chatmessage, sender, language, channelstring, target, flags, zoneid, channelnumber, channelname, _, counter, guid, ...)
	local message = nil
	local pattern, lmsg = nil, nil
	local omsg, classtxt = nil, nil
	local lfmclasses = {"dk", "dr", "hu", "ma", "pa", "pr", "ro", "sh", "wa", "wr"}
	local profs = {"jc", "enc", "lw", "ins", "tlr", "bs", "alc", "eng", "misc"}
	local misc = {"lfg", "wtb", "lf", "wts", "lfw"}
	local matched, lfmgen = false, true
	local channelmatch = false
	local tmp = {}
	local i, channels = {}
	local tfwidx, tfwname, tfwmsg, tfwcolour
	local lfmsender, original

	lfmsender = sender

	if LFM.db.char.channels.general then
		if zoneid == 1 or event == "CHAT_MSG_SAY" or event == "CHAT_MSG_YELL" then channelmatch = true end
	end

	if LFM.db.char.channels.trade then
		if zoneid == 2 then channelmatch = true end
	end

	if LFM.db.char.channels.guild then
		if event == "CHAT_MSG_GUILD" then channelmatch = true end
	end

	if LFM.db.char.channels.lfg then
		if zoneid == 26 then channelmatch = true end
	end

	if not channelmatch then
		channels = {GetChannelList()}
		for i = 1, #channels, 2 do
			if channels[i] > 4 then
				if LFM.db.char.channels["cc" .. channels[i + 1]] then
					if channelnumber == channels[i] then channelmatch = true end
				end
			end
		end
	end

	if LFM.alertframe:IsVisible() then
		if channelmatch or LFM.testmode then
			original = chatmessage
			chatmessage = formatspecial(chatmessage)
			omsg = chatmessage
			if LFM.tfw then
				tfwidx, tfwname, tfwmsg, tfwcolor = string.match(chatmessage, LFM.TFW_PATTERN)
				if tfwmsg then
					omsg = tfwmsg
					lfmsender = tfwname
				end
			end
			if #LFM.proflist ~= 0 then
				cfflag = "PROF"
				matched = doSearch(LFM.proflist, omsg, LFM.lftxt, lfmsender, guid, original)
			end
			if not matched then
				if LFM.db.char.monitoring.lfm then
					cfflag = "LFM"
					--if #LFM.raidlist ~=0 then matched = doSearch(LFM.raidlist, omsg, LFM.lfmtxt, lfmsender, guid, original) end
					if #LFM.classlist ~=0 then
						--if (not matched) and #LFM.raidlist ~=0 then
						--	if LFM.db.profile.options.overrideraid then
						--		matched = doSearch(LFM.classlist, omsg, LFM.lfmtxt, lfmsender, guid, original)
						--	end
						--else
						matched = doSearch(LFM.classlist, omsg, LFM.lfmtxt, lfmsender, guid, original) --end
					else
						if matched then addAlert(original, lfmsender, guid) end
					end
					if (not matched) and #LFM.classlist == 0 then
						if lfmgen then matched = doSearch(LFM.nop, omsg, LFM.lfmtxt, lfmsender, guid, original) end
					end
				end
        	end
			if not matched then
				for _, other in ipairs(misc) do
					if LFM.db.char.monitoring[other] then
						cfflag = string.upper(other)
						if not matched then matched = doSearch(LFM.nop, omsg, LFM[other .. "txt"], lfmsender, guid, original) end
					end
				end
			end
			if not matched then
				if (LFM.db.profile.options.basic or "") ~= "" then
					cfflag = "CUSTOM"
					tmp = {}
					if string.find(LFM.db.profile.options.basicesc, " ") then
						tmp = {strsplit(" ", string.lower(LFM.db.profile.options.basicesc))}
					else table.insert(tmp, string.lower(LFM.db.profile.options.basicesc)) end
					matched = doSearch(LFM.nop, omsg, tmp, lfmsender, guid, original)
				end
			end
			if not matched then
				if (LFM.db.profile.options.adv or "") ~= "" then
					cfflag = "CUSTOM"
					tmp = {}
					table.insert(tmp, LFM.db.profile.options.adv)
					matched = doSearch(LFM.nop, omsg, tmp, lfmsender, guid, original)
				end
			end
		end
	end
end

function events.CHAT_MSG_CHANNEL(event, chatmessage, sender, language, channelstring, target, flags, zoneid, channelnumber, channelname, counter, ...)
	ChatParser(event, chatmessage, sender, language, channelstring, target, flags, zoneid, channelnumber, channelname, counter, ...)
end

function events.CHAT_MSG_WHISPER_INFORM(event, chatmessage, sender, language, channelstring, target, flags, zoneid, channelnumber, channelname, counter, ...)
	ChatParser(event, chatmessage, sender, language, channelstring, target, flags, zoneid, channelnumber, channelname, counter, ...)
end

function events.CHAT_MSG_YELL(event, chatmessage, sender, language, channelstring, target, flags, zoneid, channelnumber, channelname, counter, ...)
	ChatParser(event, chatmessage, sender, language, channelstring, target, flags, zoneid, channelnumber, channelname, counter, ...)
end

function events.CHAT_MSG_SAY(event, chatmessage, sender, language, channelstring, target, flags, zoneid, channelnumber, channelname, counter, ...)
	ChatParser(event, chatmessage, sender, language, channelstring, target, flags, zoneid, channelnumber, channelname, counter, ...)
end

function events.CHAT_MSG_CHANNEL_NOTICE(event, chatmessage, sender, language, channelstring, target, flags, zoneid, channelnumber, channelname, counter, ...)
	tradechannel = LFM:GetChannelNumber("Trade")
	LFM:DisableCheck()
end

local function LFM_DDMenuItem_OnClick(this, button, down)
	local opt = this.value
	local reply, rtmp, cl

	PlaySound("igMainMenuOption")

	if opt == 1 then
		reply = GetAutoReply()
		if (reply or "") == "" then StaticPopup_Show("LFM_NOAUTOREPLY")
		else
			rtmp = string.sub(reply, 1, 255)
			sendWhisper("NORMAL", "LFM", reply, "WHISPER", nil, pname)
		end
	elseif opt == 2 then ChatFrame1EditBox:SetText(LFM.WHISPER_COMMAND .. " " .. pname .. " ")
	elseif opt == 3 then AddIgnore(pname)
	elseif opt == 4 then --cancel
	end
end

local function LFM_DD_Initialise()
	local info
	info = {}
	info.text = pname
	info.value = 0
	info.owner = ddmenu
	info.isTitle = true
	info.func = nil
	info.notCheckable = 1
	UIDropDownMenu_AddButton(info)

	info = {}
	info.text = LFM.AUTOREPLY
	info.value = 1
	info.owner = ddmenu
	info.notCheckable = 1
	info.isTitle = nil

	if pflag == "LFG" or pflag == "WTB" then info.disabled = true
	else info.disabled = nil end

	info.tooltipTitle = LFM.AUTOREPLY
	info.tooltipText = string.format(LFM.SEND_AUTO, pname)
	info.func = LFM_DDMenuItem_OnClick
	UIDropDownMenu_AddButton(info, 1)

	info = {}
	info.text = _G.WHISPER
	info.value = 2
	info.owner = ddmenu
	info.isTitle = nil
	info.disabled = nil
	info.notCheckable = 1
	info.tooltipTitle = _G.WHISPER
	info.tooltipText = string.format(LFM.WHISPER, pname)
	info.func = LFM_DDMenuItem_OnClick
	UIDropDownMenu_AddButton(info, 1)

	info = {}
	info.text = _G.IGNORE
	info.value = 3
	info.owner = ddmenu
	info.isTitle = nil
	info.disabled = nil
	info.tooltipTitle = _G.IGNORE
	info.notCheckable = 1
	info.tooltipText = string.format(LFM.IGNORE_LIST, pname)
	info.func = LFM_DDMenuItem_OnClick
	UIDropDownMenu_AddButton(info, 1)

	info = {}
	info.text = _G.CANCEL
	info.value = 4
	info.owner = ddmenu
	info.isTitle = nil
	info.disabled = nil
	info.notCheckable = 1
	info.tooltipTitle = _G.CANCEL
	info.tooltipText = LFM.CLOSE_MENU
	info.func = LFM_DDMenuItem_OnClick
	UIDropDownMenu_AddButton(info, 1)
end

local function LFM_ShowDDMenu(link)
	local htype, hname, hmod, flag = strsplit(":", link)
	pname = hname
	pflag = flag
	ddmenu = _G.LFMDDMenu
	ddmenu.initialize = LFM_DD_Initialise
	ddmenu.displayMode = "MENU"
	ddmenu.name = "LFM"
	PlaySound("igMainMenuOpen")
	ToggleDropDownMenu(1, nil, ddmenu, "cursor")
end

--Hook SetItemRef so LFM can handle its own custom links
local ORIG_SetItemRef = _G.SetItemRef

local function LFM_SetItemRef(link, text, button, ...)
	local htype, hname, hmod, flag = strsplit(":", link)
	local runorig = nil
	if IsModifiedClick("CHATLINK") then runorig = true
	elseif button == "RightButton" then
		if hmod == "LFM" then LFM_ShowDDMenu(link)
		else runorig = true end
	else runorig = true end
	if runorig then ORIG_SetItemRef(link, text, button, ...) end
end

_G.SetItemRef = LFM_SetItemRef

--Hook ChatEdit_GetActiveWindow so LFM can capture links in the options frame
local orig_ChatEdit_GetActiveWindow = ChatEdit_GetActiveWindow

function ChatEdit_GetActiveWindow() return LFM.EditFocus or orig_ChatEdit_GetActiveWindow() end

local function merge(t1, t2)
	for _, txt in ipairs(t2) do table.insert(t1, txt) end
	return ctable
end

--build search tables
function LFM:BuildTables()
	local profs = {"jc", "enc", "lw", "ins", "tlr", "bs", "alc", "eng", "misc"}
	local prof, proflist, raidlist = nil, {}, {}
	local tank, healer, melee, ranged, cc
	local classlist = merge({}, LFM.ALL_TXT)
	local p, c, rl
	
	for _, prof in ipairs(profs) do
		if LFM.db.char.professions[prof] then proflist = merge(proflist, LFM[prof .. "txt"]) end
	end
	LFM.proflist = proflist
--	for p, c in pairs(LFM.RAIDS) do
		--if (LFMDB.raids[c.rname] == 1) then
					
			--raidlist = merge(raidlist, c.patt)
			
		--end
		
	--end
	if LFM.db.char.classes.dk then
		classlist = merge(classlist, LFM.DK_TXT)
		dps = true
		tank = true
		melee = true
	end

	if LFM.db.char.classes.dr then
		classlist = merge(classlist, LFM.DR_TXT)
		tank = true
		healer = true
		melee = true
		cc = true
	end

	if LFM.db.char.classes.hu then
		classlist = merge(classlist, LFM.HU_TXT)
		ranged = true
		cc = true
	end

	if LFM.db.char.classes.ma then
		classlist = merge(classlist, LFM.MA_TXT)
		ranged = true
		cc = true
	end

	if LFM.db.char.classes.mo then
		classlist = merge(classlist, LFM.MO_TXT)
		ranged = true
		cc = true
	end

	if LFM.db.char.classes.pa then
		classlist = merge(classlist, LFM.PA_TXT)
		tank = true
		healer = true
		melee = true
		cc = true
	end

	if LFM.db.char.classes.pr then
		classlist = merge(classlist, LFM.PR_TXT)
		healer = true
		ranged = true
		cc = true
	end

	if LFM.db.char.classes.ro then
		classlist = merge(classlist, LFM.RO_TXT)
		melee = true
		cc = true
	end

	if LFM.db.char.classes.sh then
		classlist = merge(classlist, LFM.SH_TXT)
		healer = true
		melee = true
		ranged = true
		cc = true
	end

	if LFM.db.char.classes.wa then
		classlist = merge(classlist, LFM.WA_TXT)
		ranged = true
	end

	if LFM.db.char.classes.wr then
		classlist = merge(classlist, LFM.WR_TXT)
		tank = true
		melee = true
		cc = true
	end

	if tank then classlist = merge(classlist, LFM.TANK_TXT) end
	if healer then classlist = merge(classlist, LFM.HEALER_TXT) end
	if melee then classlist = merge(classlist, LFM.MELEE_TXT) end
	if ranged then classlist = merge(classlist, LFM.RANGED_TXT) end
	if cc then classlist = merge(classlist, LFM.CC_TXT) end
	if melee or ranged then classlist = merge(classlist, LFM.DPS_TXT) end

	--LFM.raidlist = raidlist
	LFM.classlist = classlist
end